@@ -3,18 +3,18 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@nmap.com).  Dozens of software      *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
+# * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
 # * Note that the GPL places important restrictions on "derivative works",  *
 # * yet it does not provide a detailed definition of that term.  To avoid   *
@@ -56,12 +56,19 @@
 # * particularly including the GPL Section 3 requirements of providing      *
 # * source code and allowing free redistribution of the work as a whole.    *
 # *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
 # * linked combinations including the two.                                  *
 # *                                                                         *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
 # * Any redistribution of Covered Software, including any derived works,    *
 # * must obey and carry forward all of the terms of this license, including *
 # * obeying all GPL rules and restrictions.  For example, source code of    *
@@ -80,12 +87,12 @@
 # * Covered Software without special permission from the copyright holders. *
 # *                                                                         *
 # * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
+# * Nmap in other works, we are happy to help.  As mentioned above, we also *
+# * offer an alternative license to integrate Nmap into proprietary         *
 # * applications and appliances.  These contracts have been sold to dozens  *
 # * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@nmap.com for further *
+# * as providing support and updates.  They also fund the continued         *
+# * development of Nmap.  Please email sales@nmap.com for further           *
 # * information.                                                            *
 # *                                                                         *
 # * If you have received a written license agreement or contract for        *
@@ -94,20 +101,19 @@
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
 # * to the dev@nmap.org mailing list for possible incorporation into the    *
 # * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
 # * occasionally relicense the code to third parties as discussed above.    *
 # * If you wish to specify special license conditions of your               *
 # * contributions, just say so when you send them.                          *
@@ -116,7 +122,7 @@
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 # * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
 # * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -125,6 +131,12 @@ import gtk
 import sys
 import os
 from os.path import split, isfile, join, abspath, exists
+import errno
+
+# Prevent loading PyXML
+import xml
+xml.__path__ = [x for x in xml.__path__ if "_xmlplus" not in x]
+
 import xml.sax.saxutils
 
 from time import time
@@ -151,7 +163,7 @@ from zenmapCore.RecentScans import recent_scans
 from zenmapCore.UmitLogging import log
 import zenmapCore.I18N
 import zenmapGUI.Print
-from zenmapCore.UmitConf import SearchConfig, is_maemo
+from zenmapCore.UmitConf import SearchConfig, is_maemo, WindowConfig, config_parser
 from zenmapCore.NetworkInventory import FilteredNetworkInventory
 
 UmitScanWindow = None
@@ -190,8 +202,12 @@ def can_print():
 class ScanWindow(UmitScanWindow):
     def __init__(self):
         UmitScanWindow.__init__(self)
+
+        window = WindowConfig()
+
         self.set_title(_(APP_DISPLAY_NAME))
-        self.set_default_size(-1, 650)
+        self.move(window.x, window.y)
+        self.set_default_size(window.width, window.height)
 
         self.scan_interface = ScanInterface()
 
@@ -238,7 +254,7 @@ class ScanWindow(UmitScanWindow):
         except:
             about_icon = None
 
-        self.main_actions = [ \
+        self.main_actions = [
             # Top level
             ('Scan', None, _('Sc_an'), None),
 
@@ -572,7 +588,7 @@ class ScanWindow(UmitScanWindow):
         if widget is None:
             # Don't have a Print menu item for lack of support.
             return
-        entry = self.scan_interface.scan_result.scan_result_notebook.nmap_output.get_active_entry()
+        entry = self.scan_interface.scan_result.scan_result_notebook.nmap_output.get_active_entry()  # noqa
         widget.set_sensitive(entry is not None)
 
     def _load_recent_scan(self, widget):
@@ -751,9 +767,18 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             scan_interface.saved_filename = directory
 
             # Saving recent scan information
-            for filename in filenames:
-                recent_scans.add_recent_scan(filename)
-            recent_scans.save()
+            try:
+                for filename in filenames:
+                    recent_scans.add_recent_scan(filename)
+                recent_scans.save()
+            except (OSError, IOError), e:
+                alert = HIGAlertDialog(
+                        message_format=_(
+                            "Can't save recent scan information"),
+                        secondary_text=_(
+                            "Can't open file to write.\n%s") % str(e))
+                alert.run()
+                alert.destroy()
 
     def _save(self, scan_interface, saved_filename, selected_index,
             format="xml"):
@@ -763,8 +788,7 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
         try:
             scan_interface.inventory.save_to_file(
                     saved_filename, selected_index, format)
-            scan_interface.inventory.get_scans()[selected_index].unsaved = \
-                    False
+            scan_interface.inventory.get_scans()[selected_index].unsaved = False  # noqa
         except (OSError, IOError), e:
             alert = HIGAlertDialog(
                     message_format=_("Can't save file"),
@@ -779,8 +803,17 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
 
             if format == "xml":
                 # Saving recent scan information
-                recent_scans.add_recent_scan(saved_filename)
-                recent_scans.save()
+                try:
+                    recent_scans.add_recent_scan(saved_filename)
+                    recent_scans.save()
+                except (OSError, IOError), e:
+                    alert = HIGAlertDialog(
+                            message_format=_(
+                                "Can't save recent scan information"),
+                            secondary_text=_(
+                                "Can't open file to write.\n%s") % str(e))
+                    alert.run()
+                    alert.destroy()
 
     def get_empty_interface(self):
         """Return this window if it is empty, otherwise create and return a new
@@ -897,13 +930,28 @@ This scan has not been run yet. Start the scan with the "Scan" button first.'))
             elif response == gtk.RESPONSE_CANCEL:
                 return True
 
+        window = WindowConfig()
+        window.x, window.y = self.get_position()
+        window.width, window.height = self.get_size()
+        window.save_changes()
+        if config_parser.failed:
+            alert = HIGAlertDialog(
+                    message_format=_("Can't save Zenmap configuration"),
+                    # newline before path to help avoid weird line wrapping
+                    secondary_text=_(
+                        'An error occurred when saving to\n%s'
+                        '\nThe error was: %s.'
+                        ) % (Path.user_config_file, config_parser.failed))
+            alert.run()
+            alert.destroy()
+
         self.destroy()
 
         return False
 
     def _print_cb(self, *args):
         """Show a print dialog."""
-        entry = self.scan_interface.scan_result.scan_result_notebook.nmap_output.get_active_entry()
+        entry = self.scan_interface.scan_result.scan_result_notebook.nmap_output.get_active_entry()  # noqa
         if entry is None:
             return False
         zenmapGUI.Print.run_print_operation(
