@@ -3,18 +3,18 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2014 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@nmap.com).  Dozens of software      *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
+# * The Nmap Security Scanner is (C) 1996-2019 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
 # * Note that the GPL places important restrictions on "derivative works",  *
 # * yet it does not provide a detailed definition of that term.  To avoid   *
@@ -56,12 +56,19 @@
 # * particularly including the GPL Section 3 requirements of providing      *
 # * source code and allowing free redistribution of the work as a whole.    *
 # *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
 # * linked combinations including the two.                                  *
 # *                                                                         *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
 # * Any redistribution of Covered Software, including any derived works,    *
 # * must obey and carry forward all of the terms of this license, including *
 # * obeying all GPL rules and restrictions.  For example, source code of    *
@@ -80,12 +87,12 @@
 # * Covered Software without special permission from the copyright holders. *
 # *                                                                         *
 # * If you have any questions about the licensing restrictions on using     *
-# * Nmap in other works, are happy to help.  As mentioned above, we also    *
-# * offer alternative license to integrate Nmap into proprietary            *
+# * Nmap in other works, we are happy to help.  As mentioned above, we also *
+# * offer an alternative license to integrate Nmap into proprietary         *
 # * applications and appliances.  These contracts have been sold to dozens  *
 # * of software vendors, and generally include a perpetual license as well  *
-# * as providing for priority support and updates.  They also fund the      *
-# * continued development of Nmap.  Please email sales@nmap.com for further *
+# * as providing support and updates.  They also fund the continued         *
+# * development of Nmap.  Please email sales@nmap.com for further           *
 # * information.                                                            *
 # *                                                                         *
 # * If you have received a written license agreement or contract for        *
@@ -94,20 +101,19 @@
 # *                                                                         *
 # * Source is provided to this software because we believe users have a     *
 # * right to know exactly what a program is going to do before they run it. *
-# * This also allows you to audit the software for security holes (none     *
-# * have been found so far).                                                *
+# * This also allows you to audit the software for security holes.          *
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
 # * to the dev@nmap.org mailing list for possible incorporation into the    *
 # * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
 # * occasionally relicense the code to third parties as discussed above.    *
 # * If you wish to specify special license conditions of your               *
 # * contributions, just say so when you send them.                          *
@@ -116,7 +122,7 @@
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 # * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
 # * license file for more details (it's in a COPYING file included with     *
-# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -125,6 +131,7 @@ import gtk
 import gobject
 import os
 import time
+import sys
 
 # Prevent loading PyXML
 import xml
@@ -324,7 +331,7 @@ class ScanInterface(HIGVBox):
 
     def go_to_host(self, hostname):
         """Scroll the text output to the appearance of the named host."""
-        self.scan_result.scan_result_notebook.nmap_output.nmap_output.go_to_host(hostname)
+        self.scan_result.scan_result_notebook.nmap_output.nmap_output.go_to_host(hostname)  # noqa
 
     def __create_toolbar(self):
         self.toolbar = ScanToolbar()
@@ -457,7 +464,7 @@ class ScanInterface(HIGVBox):
     def update_cancel_button(self):
         """Make the Cancel button sensitive or not depending on whether the
         currently displayed scan is running."""
-        entry = self.scan_result.scan_result_notebook.nmap_output.get_active_entry()
+        entry = self.scan_result.scan_result_notebook.nmap_output.get_active_entry()  # noqa
         if entry is None:
             self.toolbar.cancel_button.set_sensitive(False)
         else:
@@ -465,7 +472,7 @@ class ScanInterface(HIGVBox):
 
     def _cancel_scan_cb(self, widget):
         """Cancel the scan whose output is shown."""
-        entry = self.scan_result.scan_result_notebook.nmap_output.get_active_entry()
+        entry = self.scan_result.scan_result_notebook.nmap_output.get_active_entry()  # noqa
         if entry is not None and entry.running:
             self.cancel_scan(entry.command)
 
@@ -473,14 +480,14 @@ class ScanInterface(HIGVBox):
         """This is like _cancel_scan_cb, but it cancels the scans that are
         currently selected in the scans list, not the one whose output is
         currently shown."""
-        model, selection = self.scan_result.scan_result_notebook.scans_list.scans_list.get_selection().get_selected_rows()
+        model, selection = self.scan_result.scan_result_notebook.scans_list.scans_list.get_selection().get_selected_rows()  # noqa
         for path in selection:
             entry = model.get_value(model.get_iter(path), 0)
             if entry.running:
                 self.cancel_scan(entry.command)
 
     def _remove_scan_cb(self, widget):
-        model, selection = self.scan_result.scan_result_notebook.scans_list.scans_list.get_selection().get_selected_rows()
+        model, selection = self.scan_result.scan_result_notebook.scans_list.scans_list.get_selection().get_selected_rows()  # noqa
         selected_refs = []
         for path in selection:
             # Kill running scans and remove finished scans from the inventory.
@@ -529,36 +536,48 @@ class ScanInterface(HIGVBox):
         """Run the given Nmap command. Add it to the list of running scans.
         Schedule a timer to refresh the output and check the scan for
         completion."""
-        command_execution = NmapCommand(command)
+        try:
+            command_execution = NmapCommand(command)
+        except IOError, e:
+            warn_dialog = HIGAlertDialog(
+                        message_format=_("Error building command"),
+                        secondary_text=_("Error message: %s") % str(e),
+                        type=gtk.MESSAGE_ERROR)
+            warn_dialog.run()
+            warn_dialog.destroy()
+            return
         command_execution.profile = profile
 
         try:
             command_execution.run_scan()
         except Exception, e:
-            text = str(e)
+            text = unicode(e)
             if isinstance(e, OSError):
                 # Handle ENOENT specially.
                 if e.errno == errno.ENOENT:
                     # nmap_command_path comes from zenmapCore.NmapCommand.
-                    text += "\n\n%s\n\n%s" % (
-                            _("This means that the nmap executable was "
-                                "not found in your system PATH, which is"),
-                            os.getenv("PATH", _("<undefined>"))
-                            )
                     path_env = os.getenv("PATH")
                     if path_env is None:
                         default_paths = []
                     else:
+                        fsencoding = sys.getfilesystemencoding()
+                        if fsencoding:
+                            path_env = path_env.decode(fsencoding, 'replace')
                         default_paths = path_env.split(os.pathsep)
+                    text += u"\n\n{}\n\n{}".format(
+                            _("This means that the nmap executable was "
+                                "not found in your system PATH, which is"),
+                            path_env or _("<undefined>")
+                            )
                     extra_paths = get_extra_executable_search_paths()
                     extra_paths = [p for p in extra_paths if (
                         p not in default_paths)]
                     if len(extra_paths) > 0:
                         if len(extra_paths) == 1:
-                            text += "\n\n" + _("plus the extra directory")
+                            text += u"\n\n" + _("plus the extra directory")
                         else:
-                            text += "\n\n" + _("plus the extra directories")
-                        text += "\n\n" + os.pathsep.join(extra_paths)
+                            text += u"\n\n" + _("plus the extra directories")
+                        text += u"\n\n" + os.pathsep.join(extra_paths)
             warn_dialog = HIGAlertDialog(
                 message_format=_("Error executing command"),
                 secondary_text=text, type=gtk.MESSAGE_ERROR)
@@ -654,7 +673,10 @@ class ScanInterface(HIGVBox):
                 warn_dialog.destroy()
         parsed.set_xml_is_temp(command.xml_is_temp)
         self.collect_umit_info(command, parsed)
-        parsed.nmap_output = command.get_output()
+        try:
+            parsed.nmap_output = command.get_output()
+        except MemoryError:
+            self.scan_result.scan_result_notebook.nmap_output.nmap_output.show_large_output_message(command)  # noqa
         self.update_ui()
         self.scans_store.finish_running_scan(command, parsed)
 
@@ -829,12 +851,19 @@ class ScanInterface(HIGVBox):
         """Sets the comment on a host from the contents of the comment text
         entry."""
         buff = widget.get_buffer()
-        host.comment = buff.get_text(
+        comment = buff.get_text(
                 buff.get_start_iter(), buff.get_end_iter())
+        if host.comment == comment:
+            # no change, ignore
+            return
+        host.comment = comment
         for scan in self.inventory.get_scans():
-            if host in scan.get_hosts():
-                scan.unsaved = True
-                break
+            for h in scan.get_hosts():
+                if (h.get_ip() == host.get_ip() and
+                        h.get_ipv6() == host.get_ipv6()):
+                    h.set_comment(host.comment)
+                    scan.unsaved = True
+                    break
 
     def build_host_details(self, hosts):
         """Builds and returns a list of "Host Details" pages corresponding to
@@ -921,10 +950,10 @@ class ScanResult(gtk.HPaned):
         self.pack2(self.scan_result_notebook, True, False)
 
     def set_nmap_output(self, msg):
-        self.scan_result_notebook.nmap_output.nmap_output.text_view.get_buffer().set_text(msg)
+        self.scan_result_notebook.nmap_output.nmap_output.text_view.get_buffer().set_text(msg)  # noqa
 
     def clear_nmap_output(self):
-        self.scan_result_notebook.nmap_output.nmap_output.text_view.get_buffer().set_text("")
+        self.scan_result_notebook.nmap_output.nmap_output.text_view.get_buffer().set_text("")  # noqa
 
     def get_host_selection(self):
         return self.scan_host_view.host_view.get_selection()
